@@ -49,6 +49,21 @@ def test_opaque() -> None:
                 assert type(frame.opaque) is tuple and len(frame.opaque) == 2
 
 
+def test_frame_duration_matches_packet() -> None:
+    with av.open(fate_suite("h264/interlaced_crop.mp4")) as container:
+        packet_durations = [
+            (p.pts, p.duration) for p in container.demux() if p.pts is not None
+        ]
+        packet_durations.sort(key=lambda x: x[0])
+
+    with av.open(fate_suite("h264/interlaced_crop.mp4")) as container:
+        frame_durations = [(f.pts, f.duration) for f in container.decode(video=0)]
+        frame_durations.sort(key=lambda x: x[0])
+
+    assert len(packet_durations) == len(frame_durations)
+    assert all(pd[1] == fd[1] for pd, fd in zip(packet_durations, frame_durations))
+
+
 def test_invalid_pixel_format() -> None:
     with pytest.raises(ValueError, match="not a pixel format: '__unknown_pix_fmt'"):
         VideoFrame(640, 480, "__unknown_pix_fmt")
@@ -209,6 +224,7 @@ def test_ndarray_gray() -> None:
     array = numpy.random.randint(0, 256, size=(480, 640), dtype=numpy.uint8)
     for format in ("gray", "gray8"):
         frame = VideoFrame.from_ndarray(array, format=format)
+        assert format in av.video.frame.supported_np_pix_fmts
         assert frame.width == 640 and frame.height == 480
         assert frame.format.name == "gray"
         assertNdarraysEqual(frame.to_ndarray(), array)
@@ -218,6 +234,7 @@ def test_ndarray_gray_align() -> None:
     array = numpy.random.randint(0, 256, size=(238, 318), dtype=numpy.uint8)
     for format in ("gray", "gray8"):
         frame = VideoFrame.from_ndarray(array, format=format)
+        assert format in av.video.frame.supported_np_pix_fmts
         assert frame.width == 318 and frame.height == 238
         assert frame.format.name == "gray"
         assertNdarraysEqual(frame.to_ndarray(), array)
@@ -227,6 +244,7 @@ def test_ndarray_grayf32() -> None:
     array = numpy.random.random_sample(size=(480, 640)).astype(numpy.float32)
     for format in ("grayf32be", "grayf32le"):
         frame = VideoFrame.from_ndarray(array, format=format)
+        assert format in av.video.frame.supported_np_pix_fmts
         assert frame.width == 640 and frame.height == 480
         assert frame.format.name == format
         assertNdarraysEqual(frame.to_ndarray(), array)
@@ -236,6 +254,7 @@ def test_ndarray_grayf32_align() -> None:
     array = numpy.random.random_sample(size=(238, 318)).astype(numpy.float32)
     for format in ("grayf32be", "grayf32le"):
         frame = VideoFrame.from_ndarray(array, format=format)
+        assert format in av.video.frame.supported_np_pix_fmts
         assert frame.width == 318 and frame.height == 238
         assert frame.format.name == format
         assertNdarraysEqual(frame.to_ndarray(), array)
@@ -245,6 +264,7 @@ def test_ndarray_rgb() -> None:
     array = numpy.random.randint(0, 256, size=(480, 640, 3), dtype=numpy.uint8)
     for format in ("rgb24", "bgr24"):
         frame = VideoFrame.from_ndarray(array, format=format)
+        assert format in av.video.frame.supported_np_pix_fmts
         assert frame.width == 640 and frame.height == 480
         assert frame.format.name == format
         assertNdarraysEqual(frame.to_ndarray(), array)
@@ -254,6 +274,7 @@ def test_ndarray_rgb_align() -> None:
     array = numpy.random.randint(0, 256, size=(238, 318, 3), dtype=numpy.uint8)
     for format in ("rgb24", "bgr24"):
         frame = VideoFrame.from_ndarray(array, format=format)
+        assert format in av.video.frame.supported_np_pix_fmts
         assert frame.width == 318 and frame.height == 238
         assert frame.format.name == format
         assertNdarraysEqual(frame.to_ndarray(), array)
@@ -263,6 +284,7 @@ def test_ndarray_rgba() -> None:
     array = numpy.random.randint(0, 256, size=(480, 640, 4), dtype=numpy.uint8)
     for format in ("argb", "rgba", "abgr", "bgra"):
         frame = VideoFrame.from_ndarray(array, format=format)
+        assert format in av.video.frame.supported_np_pix_fmts
         assert frame.width == 640 and frame.height == 480
         assert frame.format.name == format
         assertNdarraysEqual(frame.to_ndarray(), array)
@@ -272,11 +294,41 @@ def test_ndarray_rgba_align() -> None:
     array = numpy.random.randint(0, 256, size=(238, 318, 4), dtype=numpy.uint8)
     for format in ("argb", "rgba", "abgr", "bgra"):
         frame = VideoFrame.from_ndarray(array, format=format)
+        assert format in av.video.frame.supported_np_pix_fmts
         assert frame.width == 318 and frame.height == 238
         assert frame.format.name == format
         assertNdarraysEqual(frame.to_ndarray(), array)
 
 
+def test_ndarray_bayer8() -> None:
+    array = numpy.random.randint(0, 256, size=(480, 640), dtype=numpy.uint8)
+    for format in ("bayer_bggr8", "bayer_gbrg8", "bayer_grbg8", "bayer_rggb8"):
+        frame = VideoFrame.from_ndarray(array, format=format)
+        assert format in av.video.frame.supported_np_pix_fmts
+        assert frame.width == 640 and frame.height == 480
+        assert frame.format.name == format
+        assertNdarraysEqual(frame.to_ndarray(), array)
+
+
+def test_ndarray_bayer16() -> None:
+    array = numpy.random.randint(0, 65536, size=(480, 640), dtype=numpy.uint16)
+    for format in (
+        "bayer_bggr16be",
+        "bayer_bggr16le",
+        "bayer_gbrg16be",
+        "bayer_gbrg16le",
+        "bayer_grbg16be",
+        "bayer_grbg16le",
+        "bayer_rggb16be",
+        "bayer_rggb16le",
+    ):
+        frame = VideoFrame.from_ndarray(array, format=format)
+        assert format in av.video.frame.supported_np_pix_fmts
+        assert frame.width == 640 and frame.height == 480
+        assert frame.format.name == format
+        assertNdarraysEqual(frame.to_ndarray(), array)
+
+
 def test_ndarray_gbrp() -> None:
     array = numpy.random.randint(0, 256, size=(480, 640, 3), dtype=numpy.uint8)
     frame = VideoFrame.from_ndarray(array, format="gbrp")
@@ -297,6 +349,7 @@ def test_ndarray_gbrp10() -> None:
     array = numpy.random.randint(0, 1024, size=(480, 640, 3), dtype=numpy.uint16)
     for format in ("gbrp10be", "gbrp10le"):
         frame = VideoFrame.from_ndarray(array, format=format)
+        assert format in av.video.frame.supported_np_pix_fmts
         assert frame.width == 640 and frame.height == 480
         assert frame.format.name == format
         assertNdarraysEqual(frame.to_ndarray(), array)
@@ -306,6 +359,7 @@ def test_ndarray_gbrp10_align() -> None:
     array = numpy.random.randint(0, 1024, size=(238, 318, 3), dtype=numpy.uint16)
     for format in ("gbrp10be", "gbrp10le"):
         frame = VideoFrame.from_ndarray(array, format=format)
+        assert format in av.video.frame.supported_np_pix_fmts
         assert frame.width == 318 and frame.height == 238
         assert frame.format.name == format
         assertNdarraysEqual(frame.to_ndarray(), array)
@@ -315,6 +369,7 @@ def test_ndarray_gbrp12() -> None:
     array = numpy.random.randint(0, 4096, size=(480, 640, 3), dtype=numpy.uint16)
     for format in ("gbrp12be", "gbrp12le"):
         frame = VideoFrame.from_ndarray(array, format=format)
+        assert format in av.video.frame.supported_np_pix_fmts
         assert frame.width == 640 and frame.height == 480
         assert frame.format.name == format
         assertNdarraysEqual(frame.to_ndarray(), array)
@@ -324,6 +379,7 @@ def test_ndarray_gbrp12_align() -> None:
     array = numpy.random.randint(0, 4096, size=(238, 318, 3), dtype=numpy.uint16)
     for format in ("gbrp12be", "gbrp12le"):
         frame = VideoFrame.from_ndarray(array, format=format)
+        assert format in av.video.frame.supported_np_pix_fmts
         assert frame.width == 318 and frame.height == 238
         assert frame.format.name == format
         assertNdarraysEqual(frame.to_ndarray(), array)
@@ -333,6 +389,7 @@ def test_ndarray_gbrp14() -> None:
     array = numpy.random.randint(0, 16384, size=(480, 640, 3), dtype=numpy.uint16)
     for format in ("gbrp14be", "gbrp14le"):
         frame = VideoFrame.from_ndarray(array, format=format)
+        assert format in av.video.frame.supported_np_pix_fmts
         assert frame.width == 640 and frame.height == 480
         assert frame.format.name == format
         assertNdarraysEqual(frame.to_ndarray(), array)
@@ -342,6 +399,7 @@ def test_ndarray_gbrp14_align() -> None:
     array = numpy.random.randint(0, 16384, size=(238, 318, 3), dtype=numpy.uint16)
     for format in ("gbrp14be", "gbrp14le"):
         frame = VideoFrame.from_ndarray(array, format=format)
+        assert format in av.video.frame.supported_np_pix_fmts
         assert frame.width == 318 and frame.height == 238
         assert frame.format.name == format
         assertNdarraysEqual(frame.to_ndarray(), array)
@@ -351,6 +409,7 @@ def test_ndarray_gbrp16() -> None:
     array = numpy.random.randint(0, 65536, size=(480, 640, 3), dtype=numpy.uint16)
     for format in ("gbrp16be", "gbrp16le"):
         frame = VideoFrame.from_ndarray(array, format=format)
+        assert format in av.video.frame.supported_np_pix_fmts
         assert frame.width == 640 and frame.height == 480
         assert frame.format.name == format
         assertNdarraysEqual(frame.to_ndarray(), array)
@@ -360,6 +419,7 @@ def test_ndarray_gbrp16_align() -> None:
     array = numpy.random.randint(0, 65536, size=(238, 318, 3), dtype=numpy.uint16)
     for format in ("gbrp16be", "gbrp16le"):
         frame = VideoFrame.from_ndarray(array, format=format)
+        assert format in av.video.frame.supported_np_pix_fmts
         assert frame.width == 318 and frame.height == 238
         assert frame.format.name == format
         assertNdarraysEqual(frame.to_ndarray(), array)
@@ -469,6 +529,24 @@ def test_ndarray_yuv444p16() -> None:
         assertNdarraysEqual(frame.to_ndarray(), array)
 
 
+def test_ndarray_yuv420p10le() -> None:
+    array = numpy.random.randint(0, 65536, size=(3, 480, 640), dtype=numpy.uint16)
+    for format in ("yuv420p10le",):
+        frame = VideoFrame.from_ndarray(array, format=format)
+        assert frame.width == 640 and frame.height == 480
+        assert frame.format.name == format
+        assert format in av.video.frame.supported_np_pix_fmts
+
+
+def test_ndarray_yuv422p10le() -> None:
+    array = numpy.random.randint(0, 65536, size=(3, 480, 640), dtype=numpy.uint16)
+    for format in ("yuv422p10le",):
+        frame = VideoFrame.from_ndarray(array, format=format)
+        assert frame.width == 640 and frame.height == 480
+        assert frame.format.name == format
+        assert format in av.video.frame.supported_np_pix_fmts
+
+
 def test_ndarray_yuv444p16_align() -> None:
     array = numpy.random.randint(0, 65536, size=(238, 318, 3), dtype=numpy.uint16)
     for format in ("yuv444p16be", "yuv444p16le"):
@@ -537,6 +615,17 @@ def test_ndarray_rgb48be() -> None:
     assertPixelValue16(frame.planes[0], array[0][0][0], "big")
 
 
+def test_ndarray_bgr48be() -> None:
+    array = numpy.random.randint(0, 65536, size=(480, 640, 3), dtype=numpy.uint16)
+    frame = VideoFrame.from_ndarray(array, format="bgr48be")
+    assert frame.width == 640 and frame.height == 480
+    assert frame.format.name == "bgr48be"
+    assertNdarraysEqual(frame.to_ndarray(), array)
+
+    # check endianness by examining blue value of first pixel
+    assertPixelValue16(frame.planes[0], array[0][0][0], "big")
+
+
 def test_ndarray_rgb48le() -> None:
     array = numpy.random.randint(0, 65536, size=(480, 640, 3), dtype=numpy.uint16)
     frame = VideoFrame.from_ndarray(array, format="rgb48le")
@@ -548,6 +637,17 @@ def test_ndarray_rgb48le() -> None:
     assertPixelValue16(frame.planes[0], array[0][0][0], "little")
 
 
+def test_ndarray_bgr48le() -> None:
+    array = numpy.random.randint(0, 65536, size=(480, 640, 3), dtype=numpy.uint16)
+    frame = VideoFrame.from_ndarray(array, format="bgr48le")
+    assert frame.width == 640 and frame.height == 480
+    assert frame.format.name == "bgr48le"
+    assertNdarraysEqual(frame.to_ndarray(), array)
+
+    # check endianness by examining blue value of first pixel
+    assertPixelValue16(frame.planes[0], array[0][0][0], "little")
+
+
 def test_ndarray_rgb48le_align() -> None:
     array = numpy.random.randint(0, 65536, size=(238, 318, 3), dtype=numpy.uint16)
     frame = VideoFrame.from_ndarray(array, format="rgb48le")
@@ -559,6 +659,17 @@ def test_ndarray_rgb48le_align() -> None:
     assertPixelValue16(frame.planes[0], array[0][0][0], "little")
 
 
+def test_ndarray_bgr48le_align() -> None:
+    array = numpy.random.randint(0, 65536, size=(238, 318, 3), dtype=numpy.uint16)
+    frame = VideoFrame.from_ndarray(array, format="bgr48le")
+    assert frame.width == 318 and frame.height == 238
+    assert frame.format.name == "bgr48le"
+    assertNdarraysEqual(frame.to_ndarray(), array)
+
+    # check endianness by examining blue value of first pixel
+    assertPixelValue16(frame.planes[0], array[0][0][0], "little")
+
+
 def test_ndarray_rgba64be() -> None:
     array = numpy.random.randint(0, 65536, size=(480, 640, 4), dtype=numpy.uint16)
     frame = VideoFrame.from_ndarray(array, format="rgba64be")
@@ -570,6 +681,17 @@ def test_ndarray_rgba64be() -> None:
     assertPixelValue16(frame.planes[0], array[0][0][0], "big")
 
 
+def test_ndarray_bgra64be() -> None:
+    array = numpy.random.randint(0, 65536, size=(480, 640, 4), dtype=numpy.uint16)
+    frame = VideoFrame.from_ndarray(array, format="bgra64be")
+    assert frame.width == 640 and frame.height == 480
+    assert frame.format.name == "bgra64be"
+    assertNdarraysEqual(frame.to_ndarray(), array)
+
+    # check endianness by examining blue value of first pixel
+    assertPixelValue16(frame.planes[0], array[0][0][0], "big")
+
+
 def test_ndarray_rgba64le() -> None:
     array = numpy.random.randint(0, 65536, size=(480, 640, 4), dtype=numpy.uint16)
     frame = VideoFrame.from_ndarray(array, format="rgba64le")
@@ -581,6 +703,17 @@ def test_ndarray_rgba64le() -> None:
     assertPixelValue16(frame.planes[0], array[0][0][0], "little")
 
 
+def test_ndarray_bgra64le() -> None:
+    array = numpy.random.randint(0, 65536, size=(480, 640, 4), dtype=numpy.uint16)
+    frame = VideoFrame.from_ndarray(array, format="bgra64le")
+    assert frame.width == 640 and frame.height == 480
+    assert frame.format.name == "bgra64le"
+    assertNdarraysEqual(frame.to_ndarray(), array)
+
+    # check endianness by examining blue value of first pixel
+    assertPixelValue16(frame.planes[0], array[0][0][0], "little")
+
+
 def test_ndarray_rgb8() -> None:
     array = numpy.random.randint(0, 256, size=(480, 640), dtype=numpy.uint8)
     frame = VideoFrame.from_ndarray(array, format="rgb8")
@@ -603,6 +736,7 @@ def test_ndarray_pal8():
     frame = VideoFrame.from_ndarray((array, palette), format="pal8")
     assert frame.width == 640 and frame.height == 480
     assert frame.format.name == "pal8"
+    assert frame.format.name in av.video.frame.supported_np_pix_fmts
     returned = frame.to_ndarray()
     assert type(returned) is tuple and len(returned) == 2
     assertNdarraysEqual(returned[0], array)
@@ -614,6 +748,7 @@ def test_ndarray_nv12() -> None:
     frame = VideoFrame.from_ndarray(array, format="nv12")
     assert frame.width == 640 and frame.height == 480
     assert frame.format.name == "nv12"
+    assert frame.format.name in av.video.frame.supported_np_pix_fmts
     assertNdarraysEqual(frame.to_ndarray(), array)
 
 
@@ -769,6 +904,25 @@ def test_shares_memory_rgba() -> None:
     assertNdarraysEqual(frame.to_ndarray(), array)
 
 
+def test_shares_memory_bayer8() -> None:
+    for format in ("bayer_rggb8", "bayer_bggr8", "bayer_grbg8", "bayer_gbrg8"):
+        array = numpy.random.randint(0, 256, size=(357, 318), dtype=numpy.uint8)
+        frame = VideoFrame.from_numpy_buffer(array, format)
+        assertNdarraysEqual(frame.to_ndarray(), array)
+
+        array[...] = numpy.random.randint(0, 256, size=(357, 318), dtype=numpy.uint8)
+        assertNdarraysEqual(frame.to_ndarray(), array)
+
+        array = numpy.random.randint(0, 256, size=(357, 318), dtype=numpy.uint8)
+        array = array[:, :300]
+        assert not array.data.c_contiguous
+        frame = VideoFrame.from_numpy_buffer(array, format)
+        assertNdarraysEqual(frame.to_ndarray(), array)
+
+        array[...] = numpy.random.randint(0, 256, size=array.shape, dtype=numpy.uint8)
+        assertNdarraysEqual(frame.to_ndarray(), array)
+
+
 def test_shares_memory_yuv420p() -> None:
     array = numpy.random.randint(0, 256, size=(512 * 6 // 4, 256), dtype=numpy.uint8)
     frame = VideoFrame.from_numpy_buffer(array, "yuv420p")
